from client import client

CTG_ASSEMBLY = 52
CTG_FINISHED = 50
LOC_FINISHED = 296
LOC_FINISHED_PRODUCTION_ROOM1 = 299
LOC_SECTION_F_GOOD = 237
LOC_SECTION_F_DOOMED = 239
LOC_SECTION_H_GOOD = 202
LOC_SECTION_H_DOOMED = 206
LOC_STOCK = 11
LOC_WIP = 272
LOC_WIP_PRODUCTION_ROOM1 = 275
LOC_WIP_PRODUCTION_ROOM2 = 288
c = client()

production_2_assembly_ids = c.search(
    'product.product', 
    ['|', ('name', 'ilike', 'base assy'),
     '|', ('name', 'ilike', 'tuned stage assy'),
     '|', ('name', 'ilike', 'LSR carriage assy'),
     '|', ('default_code', '=', 'T-LSR Motor Assy'),
     '|', ('name', 'ilike', 'T-LSR Motor Contr'),
     ('name', 'ilike', 'LSM Motor Assy')])

def remove_location_path(products):
    path_ids = reduce(lambda ids, product: ids + product['path_ids'], products, [])
    c.unlink('stock.location.path', path_ids)

def add_location_path(products, default_location_id):
    for product in products:
        if not product['path_ids']:
            c.create(
                'stock.location.path', 
                {
                    'product_id': product['id'],
                    'location_from_id': LOC_STOCK,
                    'location_dest_id': default_location_id,
                    'auto': 'transparent'
                })
all_assembly_products = c.search_fetch(
    'product.product', 
    [('categ_id', 'child_of', [CTG_ASSEMBLY])],
    ['path_ids'])
finished_products = c.search_fetch(
    'product.product', 
    [('categ_id', 'child_of', [CTG_FINISHED])],
    ['path_ids'])
production_2_assembly_products = filter(
    lambda row: row['id'] in production_2_assembly_ids, all_assembly_products)
remove_location_path(
    production_2_assembly_products)
add_location_path(
    production_2_assembly_products,
    LOC_WIP_PRODUCTION_ROOM2)
add_location_path(
    filter(lambda row: row['id'] not in production_2_assembly_ids, all_assembly_products),
    LOC_WIP_PRODUCTION_ROOM1)
add_location_path(
    finished_products,
    LOC_FINISHED_PRODUCTION_ROOM1)

finished_product_location_ids = c.search(
    'stock.location',
    [('location_id', 'child_of', [LOC_FINISHED])])
wip_location_ids = c.search(
        'stock.location',
        [('location_id', 'child_of', [LOC_WIP])])
all_location_ids = (
    finished_product_location_ids + 
    wip_location_ids + 
    [
        LOC_SECTION_F_DOOMED, 
        LOC_SECTION_F_GOOD, 
        LOC_SECTION_H_DOOMED, 
        LOC_SECTION_H_GOOD])
for location_id in [LOC_FINISHED, LOC_FINISHED_PRODUCTION_ROOM1]:
    finished_product_location_ids.remove(location_id)
for location_id in [LOC_WIP, LOC_WIP_PRODUCTION_ROOM1, LOC_WIP_PRODUCTION_ROOM2]:
    wip_location_ids.remove(location_id)
    
def replace_locations(location_ids, product_ids, new_id):
    for location in c.fetch('stock.location', location_ids, ['name']):
        location_id = location['id']
        if not location['name'].startswith('doomed'):
            c.update(
                'stock.location', 
                [location_id], 
                {'name': 'doomed ' + location['name']})
        product_filter = [('product_id', 'in', product_ids)] if product_ids else []
        move_ids = c.search(
            'stock.move', 
            [('location_id', '=', location_id)] + product_filter)
        c.update('stock.move', move_ids, {'location_id': new_id})
        move_ids = c.search(
            'stock.move', 
            [('location_dest_id', '=', location_id)] + product_filter)
        c.update('stock.move', move_ids, {'location_dest_id': new_id})
        ids = c.search(
            'stock.inventory.line', 
            [('location_id', '=', location_id)] + product_filter)
        c.update('stock.inventory.line', ids, {'location_id': new_id})
        ids = c.search(
            'stock.location.path', 
            [('location_dest_id', '=', location_id)] + product_filter)
        c.update('stock.location.path', ids, {'location_dest_id': new_id})

replace_locations(finished_product_location_ids, None, LOC_FINISHED_PRODUCTION_ROOM1)
replace_locations(wip_location_ids, production_2_assembly_ids, LOC_WIP_PRODUCTION_ROOM2)
replace_locations(wip_location_ids, None, LOC_WIP_PRODUCTION_ROOM1)
replace_locations([LOC_SECTION_F_DOOMED], None, LOC_SECTION_F_GOOD)
replace_locations([LOC_SECTION_H_DOOMED], None, LOC_SECTION_H_GOOD)

rows = c.fetch('stock.location', all_location_ids, ['name'])
rows.sort(lambda x, y: cmp(x['name'], y['name']))
for row in rows:
    print "{0:10}\t{1}".format(row['id'], row['name'])
print ','.join([str(row['id']) for row in rows])

