#!/usr/bin/python

import xmlrpclib
from client import client
from string import strip
import getpass
import sys

def main():
    """
    Running migrate.py without arguments returns a list of possible databases for migration.

    Running migrate.py DATABASE1 will update DATABASE1

    Running migrate.py DATABASE1 DATABASE2 will update DATABASE1 and DATABASE2 in that order
    """

    c = client(getopts={
                      'usage':"usage: %prog [options] DATABASE1 DATABASE2 ..."
                  })
    args = c.args
    db_sock = xmlrpclib.ServerProxy(c.rpc_url+'/db')
    db_list = db_sock.list()

    if not args:
        print "Available Databases"
        print "\n".join(db_list)
        print "\nTo migrate a database, use:"
        print "migrate.py DATABASE\n"
        return

    dbs_to_migrate = map( strip, args )

    for db_name in dbs_to_migrate:
        if db_name not in db_list:
            print "Database '%s' is not a recognized database\n" % db_name
            return

    print "About to migrate the following databases:\n"
    print "\n".join(dbs_to_migrate)
    print "\nEnter master password and hit return if you are sure:"

    db_sock.migrate_databases(
      getpass.getpass(),
      dbs_to_migrate
    )

    print "\nDone. Check the logs to see that it was successful"
    
main()
