#!/usr/bin/python

""" Recreate the default minimum stock rules for any products that don't have
one.

Start by looking for all products of type 'product', then eliminate any that
already have minimum stock rules.

Finally, create the missing rules with default values.
"""

from client import client

c = client(trim_fetches=True)

warehouse_rows = c.search_fetch('stock.warehouse',
                                [],
                                ['id', 'lot_stock_id'])
assert len(warehouse_rows) == 1, 'Only a single warehouse is supported.'
warehouse_id = warehouse_rows[0]['id']
stock_location_id = warehouse_rows[0]['lot_stock_id']

products = c.search_fetch('product.product',
                          [('product_tmpl_id.type', '=', 'product')],
                          ['id', 'uom_id'],
                          context={'active_test': False})
product_map = dict([(p['id'], p['uom_id']) for p in products])

existing_rules = c.search_fetch('stock.warehouse.orderpoint', 
                                [], 
                                ['product_id'])
for rule in existing_rules:
    product_map.pop(rule['product_id'])

for product_id, uom_id in product_map.iteritems():
    c.create('stock.warehouse.orderpoint', {'location_id': stock_location_id,
                                            'product_id': product_id,
                                            'product_min_qty': 0,
                                            'product_max_qty': 0,
                                            'product_uom': uom_id,
                                            'qty_multiple': 1,
                                            'warehouse_id': warehouse_id})

print 'Created %d minimum stock rules.' % len(product_map)