#!/usr/bin/python

import csv
import datetime
import time

reports_wanted = {
    'by_ip': False,
    'by_task': True,
    'by_action': True,
    'by_object_action': True,
    'by_date': False,
    'by_max_delay': False,
    'by_time': False,
    'by_uid': False,
}

fin = open("/mnt/share/server-log/server-log-2012-02-03.txt","r")
#fin = open("/mnt/share/server-log/server-log2.txt","r")
r = csv.reader(fin)

#total,   request,net,    ip,           bytes, time,                 object, obj2,  params,        params2,cpu,ticks_idle,ticks_total, user_id?
#0.057641,0.05757,7.1e-05,192.168.1.206,13,2011-12-05 10:43:38.635739,object,execute,account.invoice,write,7.0,6365954736,7284063871

#total,   request,net,     ip,           bytes, time,                   object, obj2,  params,        params2,                                                                           cpu,ticks_idle,ticks_total, user_id?
#0.487533,0.487413,0.00012,192.168.1.102,243,2011-12-09 13:36:07.601130,wizard,execute,164,"{'model': u'mrp.production', 'report_type': 'pdf', 'id': 10335, 'form': {}, 'ids': [10335]}",48.0,6498519404,7426993571

#total,   request,net,     ip,          bytes, time,                 object, obj2,  params,  params2,cpu,ticks_idle,ticks_total, user_id
#0.014537,0.014466,7.1e-05,192.168.1.16,18,2011-12-20 06:55:44.976117,object,execute,res.lang,search,2.0,6845036070,7798790680,28

headers = [
    'total_time',
    'total_time_for_request_handling',
    'total_time_for_network_handling',
    'remote_ip',
    'byte_size',
    'request_time',
    'request_object',
    'request_object2',
    'request_params',
    'request_params2',
    'cpu_use',
    'cpu_ticks_idle',
    'cpu_ticks_total',
    'user_id',
]


by_ip   = {}
by_date = {}
by_task = {}
by_time = {}
by_uid  = {}
by_action = {}
by_object_action = {}

i = 0
for row in r:
    rec = dict(zip(headers,row))
    duration = float(rec['total_time'])
#    if duration < 0.5:
#        continue

    ip = rec['remote_ip']

    by_ip.setdefault(ip,[])
    by_ip[ip].append(duration)


    date = rec['request_time']
    date_obj = datetime.datetime.strptime(date,'%Y-%m-%d %H:%M:%S.%f')
    day = date_obj.strftime('%Y-%m-%d')
    by_date.setdefault(day,[])
    by_date[day].append(duration)

    time = date_obj.strftime('%Y-%m-%d %H:00')
    by_time.setdefault(time,[])
    by_time[time].append(duration)

    if rec['request_object'] == 'wizard':
        task = "wizard"
        action = "wizard"
    elif rec['request_object'] == 'report':
        task = "report"
        action = "report"
    else:
        task = "-".join([rec['request_object'], rec['request_object2'], rec['request_params']])
        action = rec['request_params2']
    if action in ('request_get', 'write', 'search_count', 'name_search', 'get', 'default_get'):
        object_action = action + "-" + task
    else:
        object_action = 'other'
    by_task.setdefault(task,[]).append(duration)
    by_action.setdefault(action,[]).append(duration)
    by_object_action.setdefault(object_action,[]).append(duration)

    if 'user_id' in rec and rec['user_id']:
        user_id = rec['user_id']
        by_uid.setdefault(user_id,[]).append(duration)

    i += 1

sorted_keys = sorted(by_ip.iteritems(), key=lambda x: len(x[1]), reverse=True  )

if reports_wanted['by_ip']:
    print "By IP:"
    for k,v in sorted_keys:
        v.sort()
        s = ",".join(map(lambda x:str(x), v))
        print "%s: %d - %s" % (k,float(sum(v))/len(v),s)

if reports_wanted['by_date']:
    print "\nBy Date:"
    sorted_keys = sorted(by_date.iteritems(), key=lambda x: x[0] )
    for k,v in sorted_keys:
        print "%s: %d avg(%d) max(%d)" % (k,len(v),float(sum(v))/len(v),max(v))


if reports_wanted['by_task']:
    print "\nBy Task:"
    sorted_keys = sorted(by_task.iteritems(), key=lambda x: len(x[1]), reverse=True )
    for k,v in sorted_keys:
        print "%s: %d avg(%f) max(%d)" % (k,len(v),float(sum(v))/len(v),max(v))

if reports_wanted['by_action']:
    print "\nBy Action:"
    sorted_keys = sorted(by_action.iteritems(), key=lambda x: sum(x[1]), reverse=True )
    for k,v in sorted_keys:
        v.sort()
        count = len(v)
        index25 = count // 4
        index50 = count // 2
        index75 = (3 * count) // 4
        index90 = (9 * count) // 10
        print ("%s: %d avg(%f) 25%% < %0.3fs, 50%% < %0.3fs, 75%% < %0.3fs, " +
            "90%% < %0.3fs") % (
            k,
            count,
            float(sum(v))/count,
            v[index25],
            v[index50],
            v[index75],
            v[index90])

if reports_wanted['by_object_action']:
    print "\nBy Object Action:"
    sorted_keys = sorted(by_object_action.iteritems(), key=lambda x: sum(x[1]), reverse=True )
    for k,v in sorted_keys:
        v.sort()
        count = len(v)
        index25 = count // 4
        index50 = count // 2
        index75 = (3 * count) // 4
        index90 = (9 * count) // 10
        print ("%s: %d avg(%f) 25%% < %0.3fs, 50%% < %0.3fs, 75%% < %0.3fs, " +
            "90%% < %0.3fs") % (
            k,
            count,
            float(sum(v))/count,
            v[index25],
            v[index50],
            v[index75],
            v[index90])

if reports_wanted['by_uid']:
    print "\nBy UID:"
    sorted_keys = sorted(by_uid.iteritems(), key=lambda x: len(x[1]), reverse=True )
    for k,v in sorted_keys:
        print "%s: %d avg(%d) max(%d)" % (k,len(v),float(sum(v))/len(v),max(v))

if reports_wanted['by_time']:
    print "\nBy Time:"
    sorted_keys = sorted(by_time.iteritems(), key=lambda x: x[0] )
    for k,v in sorted_keys:
        print "%s: %s avg(%d) max(%d)" % (k,"*" * (len(v)/2),float(sum(v))/len(v),max(v))

if reports_wanted['by_max_delay']:
    print "\nBy Max Delay:"
    sorted_keys = sorted(by_time.iteritems(), key=lambda x: x[0] )
    for k,v in sorted_keys:
        print "%s: %s avg(%d) max(%d) hits(%d)" % (k,"*" * int(float(sum(v))/len(v)),float(sum(v))/len(v),max(v),len(v))

print "\n%d items" % i
