#!/usr/bin/python

from client import client
#import sys

# Search for all procurements that are marked as ready and check if they should
# have been completed because of an old bug.

STOCK_LOC = 11

c = client()

#c.execute(
#    'stock.move',
#    'action_done',
#    [16925])
#c.exec_workflow('mrp.procurement', '', 989)
#sys.exit()

procurements  = c.search_fetch(
      'mrp.procurement',
      [('state', 'in', ['ready'])],
      ['move_id', 'purchase_id'])

moves_to_complete = []
procurements_to_trigger = []
warning_count = 0

def find_extra_move_to_complete(client, procurement, dummy_move):
    """ Check for the scenario where an extra move was chained in for the
    finished product of a production order.
    
    Return the move_id that should be completed, otherwise None.
    """
    
    previous_moves = client.search_fetch(
        'stock.move',
        [('move_dest_id', '=', dummy_move['id'])],
        ['state', 'production_id'])
    if len(previous_moves) == 1:
        if not previous_moves[0]['production_id']:
            previous_move_should_complete = True
            production_moves = client.search_fetch(
                'stock.move',
                [('move_dest_id', '=', previous_moves[0]['id'])],
                ['state'])
            for move in production_moves:
                if move['state'] not in ('done', 'cancel'):
                    previous_move_should_complete = False
                    print (
                        "Production move id %d for procurement id %d has state '%s'" %
                        (move['id'], procurement['id'], move['state']))
            if previous_move_should_complete and len(production_moves) == 1:
                return previous_moves[0]['id']
    print (
        "Dummy move for procurement id %d has state '%s'." % 
        (
            procurement['id'], 
            dummy_move['state']))
    return None

def find_move_to_complete(client, procurement):
    """ Check for any moves that need to be completed for the procurement.
    
    Return the move id, or None.
    """
    dummy_move = client.fetch(
        'stock.move',
        [procurement['move_id'][0]],
        ['state', 'location_id', 'location_dest_id'])[0]
    if (dummy_move['location_id'][0] != STOCK_LOC or 
        dummy_move['location_dest_id'][0] != STOCK_LOC):
        print (
            "Dummy move for procurement id %d is from %d to %d." % 
            (
                procurement['id'], 
                dummy_move['location_id'][0], 
                dummy_move['location_dest_id'][0]))
        return None
    
    if dummy_move['state'] != 'assigned':
        return find_extra_move_to_complete(client, procurement, dummy_move)
    
    move_should_complete = True
    previous_moves = client.search_fetch(
        'stock.move',
        [('move_dest_id', '=', dummy_move['id'])],
        ['state'])
    for move in previous_moves:
        if move['state'] not in ('done', 'cancel'):
            move_should_complete = False
            print (
                "Previous move id %d for procurement id %d has state '%s'" %
                (move['id'], procurement['id'], move['state']))
    
    if move_should_complete:
        return dummy_move['id']
    else:
        return None
    
for procurement in procurements:
    move_id = find_move_to_complete(c, procurement)
    if move_id:
        moves_to_complete.append(move_id)
        procurements_to_trigger.append(procurement['id'])
    else:
        warning_count += 1
        print "Procurement id %d was skipped." % procurement['id']
            
c.execute(
    'stock.move',
    'action_done',
    moves_to_complete)
for procurement_id in procurements_to_trigger:
    c.exec_workflow('mrp.procurement', '', procurement_id)

print (
    "Completed %d moves, and had %d warnings." % 
    (len(moves_to_complete), warning_count))
