#!/usr/bin/env python
# coding: utf-8

import xmlrpclib
import csv
import ConfigParser
import os

'''
Save a file like this in:

~/.zerp_config


----[ snip start ]-----------------------

[server]
connection: localhost

[server localhost]
username: user
password: pass
database: my-database-foo
rpc_url: http://localhost:8069/xmlrpc

[server localhost2]
username: user
password: pass
database: my-database-foo2
rpc_url: http://localhost:8069/xmlrpc


[server live]
prompt: This is the live server! Type Ctrl-C to abort, or Enter to continue.
username: user
password: pass
database: live
rpc_url: http://liveserver:8069/xmlrpc

----[ snip end ]-----------------------

And choose which server is active via the "connection" parameter
'''


# Get connecton parameters
config = ConfigParser.ConfigParser()
config.read(os.path.expanduser('~/.zerp_config'))
server_conn_key = "server " + config.get('server','connection')
username = config.get(server_conn_key,'username') #the user
pwd = config.get(server_conn_key,'password') #the password of the user
dbname = config.get(server_conn_key,'database') #the database

# Set this to 0 if it shouldn't update the database.
# useful for debug runs
do_update = 0

# Get the user_id
sock_common = xmlrpclib.ServerProxy(config.get(server_conn_key,'rpc_url')+'/common')
user_id = sock_common.login(dbname, username, pwd)

#replace localhost with the address of the server
sock = xmlrpclib.ServerProxy(config.get(server_conn_key,'rpc_url')+'/object')

def model_search( model, constraints, span_start=0, span_limit=100000, not_sure=0 ):
# ----------------------------------------------------------    
    search_ids = sock.execute(
        dbname, 
        user_id, 
        pwd, 
        model,
        'search', 
        constraints, 
        span_start, 
        span_limit, 
        not_sure, 
        {'lang': u'en_CA', 'tz': False})
    return search_ids

def record_fetch( model, ids, fields ):
# ----------------------------------------------------------    
    records = sock.execute(
                  dbname, user_id, pwd,
                  model, 
                  'read',
                  ids,
                  fields 
                )
    return records

def model_search_fetch( model, constraints, fields, span_start=0, span_limit=100000, not_sure=0 ):
# ----------------------------------------------------------    
    search_ids = model_search( model, constraints, span_start, span_limit, not_sure )
    if len(search_ids) == 0: return
    return record_fetch( model, search_ids, fields )

def record_update( model, ids, to_set ):
# ----------------------------------------------------------    
    res = sock.execute(
                  dbname, user_id, pwd,
                  model, 
                  'write',
                  ids,
                  to_set
                )
    return res
  

# Find the stock location named "Stock"
stock_location_id = model_search(
    'stock.location', 
    [
      ('name', '=', 'Stock'),
      ('complete_name', '=', 'Physical Locations/Zaber/Stock'),
    ])

stock_location_rec = record_fetch('stock.location',stock_location_id, [ 'name', 'complete_name' ] )

print "Stock location:"
print stock_location_rec, "\n"

# Find the location for preproduction
preproduction_location_rec = model_search_fetch(
    'stock.location', 
    [
      ('name', '=', 'Preproduction'),
    ],
    [ 'name', 'complete_name' ],
    )
preproduction_location_id = preproduction_location_rec[0]['id']
print "Preproduction location:"
print preproduction_location_rec, "\n"

# Find the location for production
production_location_rec = model_search_fetch(
    'stock.location', 
    [
      ('name', '=', 'Production'),
    ],
    [ 'name', 'complete_name' ],
    )
production_location_id = production_location_rec[0]['id']
print "Production location:"
print production_location_rec, "\n"

# Find the location for customers
customer_location_rec = model_search_fetch(
    'stock.location', 
    [
      ('name', '=', 'Customers'),
    ],
    [ 'name', 'complete_name' ],
    )
customer_location_id = customer_location_rec[0]['id']
print "Precustomer location:"
print customer_location_rec, "\n"

# Now we can find all stock moves with the initial location "Stock"
stock_move_ids = model_search(
    'stock.move', 
    [
      ('location_id',       '=', stock_location_id),
      ('location_dest_id',  '=', production_location_id), # only want stock->production records
      ('name','<>','PROC:INT:leslie'),
    ])
    

print "\nFound "+str(len(stock_move_ids))+" potential candidate stock moves to update\n\n"

# Now iterate through each of the found stock moves that
# started in "Stock" and map them a bit more sensibly
seen_stock_moves = {
  5553: 1, # this is a really weird entry
  6650: 1, # this is a really weird entry
  7603: 1, # this is stock->customers. weird
  8975: 1, # This is kinda okay, we need look at it closer though bc it 
           # may be indicative of other issues
           # stock -> altech
           # altech -> production
  9410: 1, # Finished->Customers
}
product_lookup   = {}
for stock_move_id in stock_move_ids:

    # We've already dealt with this guy
    if stock_move_id in seen_stock_moves: continue
    seen_stock_moves[stock_move_id] = 1

    # Find the name of the stock move series
    move_rec = record_fetch( 'stock.move', 
            stock_move_id,
            [ 'name', 'product_id','location_id','location_dest_id', 'state' ]
        )
    move_name = move_rec['name'];

    product_id = move_rec['product_id'][0]

    # CDs do very weird things so we're going to leave that alone for now
    if product_id == 3361:
        continue

    print "Move:", move_name, "(",stock_move_id,") Product:[",product_id,"]", move_rec['product_id'][1], "|",

    # Find the other entries related to that stock move
    move_set_ids = model_search(
        'stock.move',
        [ 
          ('name', '=', move_name ), 
          ( 'id', '!=', str(stock_move_id) ), 
          ( 'product_id.id', '=', str(product_id) ) 
        ])

    # Lookup the product's default stock location's id
    if product_id not in product_lookup:
        product_rec = model_search_fetch( 
                        'stock.location.path', 
                        [ ( 'product_id', '=',  product_id ) ], 
                        ['location_dest_id'])

        if product_rec == None: 
            print "NO DEFAULT LOC"
            continue
        product_lookup[product_id] = product_rec[0]['location_dest_id'][0]

    product_default_location_id = product_lookup[product_id]

    # Any record that is associated with more than 1 move is going to be weird, we'll leave
    # it for now
    if len(move_set_ids) != 1:
        print "WEIRD ", len(move_set_ids), " ENTRIES"
        continue

    # Temporary hack to get around a fatal error
    if move_name == 'PROD:MO/00151' and product_id == 2233: continue

    # Get the pointed reference
    seen_stock_moves[move_set_ids[0]] = 1
    ref_stock_move = record_fetch(
                        'stock.move',
                        move_set_ids[0],
                        ['product_id','location_id','location_dest_id','state'])

    # Only want records where the second move goes to the same place
    # OR where the start location was stock anyways
    if     ref_stock_move['location_id'][0] != ref_stock_move['location_dest_id'][0] \
       and ref_stock_move['location_id'][0] != stock_location_id[0]:

        print "Move Name:", move_name
        print "Move ID:", stock_move_id
        print "Move from->to:",ref_stock_move['location_id'][0], "->", ref_stock_move['location_dest_id'][0]
        print "Stock Location ID: ", stock_location_id
        continue

    # Action matrix
    #
    #             Source move state
    #          cancelled          active
    #       /-----------------+-----------------\
    # D   c |                 |                 |
    # e   a |   both cancel   |   both active   |
    # s   n |                 |                 |
    # t   c |                 |                 |
    #       |                 |                 |
    # m     |                 |                 |
    # o     |-----------------+-----------------|
    # v     |                 |                 |
    # e   a |   both cancel   |   both active   |
    #     c |                 |                 |
    # s   t |                 |                 |
    # t     |                 |                 |
    # a     |                 |                 |
    # t     \-----------------+-----------------/
    #
    source_move_state = move_rec['state']
    destination_move_state = move_rec['state']

    # If do_update = 1, then we remap the records
    if do_update != 1: 
        print "UNTOUCHED"
        continue

    # Map the first entry from: Default Location -> Preproduction
    record_update(
        'stock.move',
        stock_move_id,
        { 
            'location_id':      product_default_location_id,
            'location_dest_id': preproduction_location_id,
            'state':           source_move_state,
        })

    # Then map the second entry from Preproduct -> Final Location
    #   Preproduction -> Production
    #
    record_update(
        'stock.move',
        move_set_ids[0],
        { 
            'location_id':      preproduction_location_id,
            'location_dest_id': production_location_id,
            'state':           destination_move_state,
        },
        )

    print "OK"


