#!/usr/bin/python
# -*- coding: utf-8 -*-

from client import client
import codecs
import csv
import operator

fin = codecs.open("/mnt/share/ATX_2012.csv","r", encoding='latin1')

reader = csv.DictReader(fin)

# Load all partner names

c = client()

partners  = c.search_fetch(
    'res.partner',
    [],
    ['id', 'name'])

def levenshtein(s1, s2):
    """ Calculate how different two strings are.
    
    Returns a number representing the differences. The lower the number, the
    more similar they are.
    Source: http://en.wikibooks.org/wiki/Algorithm_implementation/Strings/Levenshtein_distance#Python
    """
    if len(s1) < len(s2):
        return levenshtein(s2, s1)
    if not s1:
        return len(s2)
 
    previous_row = xrange(len(s2) + 1)
    for i, c1 in enumerate(s1):
        current_row = [i + 1]
        for j, c2 in enumerate(s2):
            insertions = previous_row[j + 1] + 1 # j+1 instead of j since previous_row and current_row are one character longer
            deletions = current_row[j] + 1       # than s2
            substitutions = previous_row[j] + (c1 != c2)
            current_row.append(min(insertions, deletions, substitutions))
        previous_row = current_row
 
    return previous_row[-1]


SUFFIXES = [
    ' inc', 
    ', inc', 
    ' inc.', 
    ', inc.', 
    ' ltd', 
    ', ltd', 
    ' ltd.', 
    ', ltd.',
    ' llc',
    ', llc']
def simplify_name(name):
    simple = unicode(name).strip().lower()
    for suffix in SUFFIXES:
        if simple.endswith(suffix):
            simple = simple[:-len(suffix)]
            break
    return simple

for partner in partners:
    partner['simple_name'] = simplify_name(partner['name'])

new_names = []
for row in reader:
    new_name = row['Company 1'].strip()
    contact_name = row['First Name'] + ' ' + row['Last Name']
    if not new_name:
        new_name = contact_name
    simple_new_name = simplify_name(new_name)
    cutoff = len(simple_new_name)
    best_names = [('', cutoff), ('', cutoff), ('', cutoff)]
    for partner in partners:
        diff = levenshtein(simple_new_name, partner['simple_name'])
        index = 3
        while (index > 0 and diff < best_names[index-1][1]):
            index -= 1
        if index < 3:
            best_names.insert(index, (partner, diff))
            del best_names[3]
    entry = {
        'name': new_name,
        'contact': contact_name,
        'best_partners': best_names,
        'diff': best_names[0][1]
    }
    new_names.append(entry)
new_names.sort(key=operator.itemgetter('diff'))

fout = codecs.open("/mnt/share/ATX_2012_matches.csv","w", encoding='latin1')

writer = csv.writer(fout)
writer.writerow([
    'New Contact', 
    'New Partner', 
    'Best Match Partner Name', 
    'partner1_id', 
    '2nd Best Name', 
    'partner2_id', 
    '3rd Best Name', 
    'partner3_id',
    'Other Partner Name',
    'partner4_id'])
for entry in new_names:
    row = [entry['contact'], entry['name']]
    for partner in entry['best_partners']:
        row.append(partner[0]['name'])
        row.append(partner[0]['id'])
    writer.writerow(row)

print "Success."